var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};

// src/utils/JsonAutoComplete.ts
var DICTIONARY = {
  "{": "}",
  "[": "]",
  '"': '"'
};
var JsonAutoComplete = class {
  constructor() {
    this.stack = [];
    this.json = "";
    this.correctJson = "";
  }
  addChunk(chunk) {
    for (let i = 0; i < chunk.length; i++) {
      const char = chunk[i];
      const lastCharInStack = this.stack[this.stack.length - 1];
      if (char === '"' && (this.stack.length === 0 || lastCharInStack !== '"')) {
        this.stack.push(DICTIONARY[char]);
      } else if (char === '"' && lastCharInStack === '"') {
        this.stack.pop();
      } else if (DICTIONARY[char] && lastCharInStack !== '"') {
        this.stack.push(DICTIONARY[char]);
      } else if (char === lastCharInStack) {
        this.stack.pop();
      }
      this.json += char;
    }
    try {
      let currentJson = this.getPossibleCorrectJson();
      JSON.parse(this.getPossibleCorrectJson());
      this.correctJson = currentJson;
      return true;
    } catch (e) {
      return false;
    }
  }
  getCorrectJson() {
    return this.correctJson;
  }
  getPossibleCorrectJson() {
    let correctedJson = this.json;
    let stackCopy = [...this.stack];
    while (stackCopy.length > 0) {
      const closingChar = stackCopy.pop();
      correctedJson += closingChar;
    }
    return correctedJson;
  }
};

// src/utils/envUtils.ts
function isNode() {
  return typeof process !== "undefined" && process.versions != null && process.versions.node != null;
}

// src/utils/objectsUtils.ts
function isFieldSchema(schema) {
  return typeof schema === "object";
}
function handleUnexpectedDataType(data) {
  throw Error(`Unexpected type of data: ${typeof data} with value: ${JSON.stringify(data)}`);
}
function generateSchema(data, schema = {}) {
  var _a, _b, _c, _d;
  if (data === null) {
    return { type: "string" };
  }
  if (Array.isArray(data)) {
    return {
      type: "array",
      items: data.length > 0 ? generateSchema(data[0], schema[0] || {}) : { type: "string" }
    };
  }
  if (data instanceof Date) {
    return { type: "string", format: "date-time" };
  }
  if (typeof data === "object") {
    const finalJsonSchema = { type: "object", properties: {}, required: Object.keys(data) };
    for (let key in data) {
      if (schema && schema[key] !== void 0 && isFieldSchema(schema[key])) {
        finalJsonSchema.properties[key] = __spreadProps(__spreadValues({}, generateSchema(data[key], schema[key])), {
          description: (_a = schema[key]) == null ? void 0 : _a.description,
          enum: (_b = schema[key]) == null ? void 0 : _b.enum
        });
        if ((_c = schema[key]) == null ? void 0 : _c.type) {
          finalJsonSchema.properties[key].type = (_d = schema[key]) == null ? void 0 : _d.type;
        }
      } else if (typeof schema === "object" && schema[key] && typeof schema[key] === "string") {
        finalJsonSchema.properties[key] = __spreadProps(__spreadValues({}, generateSchema(data[key])), {
          description: schema[key]
        });
      } else {
        finalJsonSchema.properties[key] = generateSchema(data[key]);
      }
    }
    return finalJsonSchema;
  }
  switch (typeof data) {
    case "number":
      return { type: "number" };
    case "boolean":
      return { type: "boolean" };
    case "string":
      return { type: "string" };
    default:
      handleUnexpectedDataType(data);
      return { type: "undefined" };
  }
}
function getFunctionJsonSchema(target, name, description, fieldsSchema) {
  const schema = generateSchema(target, fieldsSchema);
  return {
    name,
    description,
    parameters: schema
  };
}

// src/utils/openAIStream.ts
import { createParser } from "eventsource-parser";
function OpenAIStream(payload, openAIApiKey) {
  return __async(this, null, function* () {
    const encoder = new TextEncoder();
    const decoder = new TextDecoder();
    const func_call = {
      name: null,
      arguments: ""
    };
    const res = yield fetch("https://api.openai.com/v1/chat/completions", {
      headers: {
        "Content-Type": "application/json",
        Authorization: `Bearer ${openAIApiKey}`
      },
      method: "POST",
      body: JSON.stringify(payload)
    });
    const stream = new ReadableStream({
      start(controller) {
        return __async(this, null, function* () {
          var _a2;
          function onParse(event) {
            var _a3;
            if (event.type === "event") {
              const data = event.data;
              if (data === "[DONE]") {
                controller.close();
                return;
              }
              try {
                const json = JSON.parse(data);
                if (json.choices && json.choices.length > 0) {
                  const choice = json.choices[0];
                  const delta = choice.delta;
                  if (delta.function_call) {
                    if (delta.function_call.name) {
                      func_call.name = delta.function_call.name;
                    }
                    if (delta.function_call.arguments) {
                      func_call.arguments += delta.function_call.arguments;
                    }
                  }
                  if (choice.finish_reason === "function_call") {
                    return;
                  }
                  const text = ((_a3 = delta == null ? void 0 : delta.function_call) == null ? void 0 : _a3.arguments) || "";
                  const queue = encoder.encode(text);
                  controller.enqueue(queue);
                }
              } catch (e) {
                controller.error(e);
              }
            }
          }
          const parser = createParser(onParse);
          const reader = (_a2 = res == null ? void 0 : res.body) == null ? void 0 : _a2.getReader();
          if (reader) {
            while (true) {
              const { done, value } = yield reader.read();
              if (done) {
                break;
              }
              parser.feed(decoder.decode(value));
            }
          }
        });
      }
    });
    return stream;
  });
}

// src/jsonGPT.ts
var JsonGPT = class {
  /**
   * The JsonGPT constructor accepts an optional target object.
   */
  constructor() {
    // Private properties with default values
    this._model = "gpt-3.5-turbo-1106";
    this._maxTokens = 3097;
    this._temperature = 0;
    this._functionName = "new_object_with_filled_data";
    this._functionDescription = "Function need all values to be filled to process.";
    this._system = "You are an AI assistant. You need to call functions to complete your tasks.";
    this._promptStartingObject = "Starting function looks like this: ```{target}```.";
    this._promptContent = "The answer should be based on this content ```{content}```.";
    if (isNode())
      this._openAIApiKey = process.env.OPENAI_API_KEY;
  }
  /**
   * Set the prompt for the AI model
   * @param prompt - The prompt string
   * @returns - The `JsonGPT` instance for method chaining
   */
  prompt(prompt) {
    if (this._prompt) {
      this._prompt = prompt;
    } else {
      this._prompt = this.addPromptForStartingObject(this._target, prompt);
    }
    return this;
  }
  /**
   * Set the name of the model.
   * It is suggested to use:
   * - "gpt-3.5-turbo-0613" (default)
   * - "gpt-4-0613"
   * - or any other model from https://platform.openai.com/docs/models/overview
   * @param name of the model
   */
  model(model) {
    this._model = model;
    return this;
  }
  /**
   * Set the OpenAI key
   * @param openAIApiKey - Your OpenAI API Key
   */
  openAIApiKey(openAIApiKey) {
    this._openAIApiKey = openAIApiKey;
    return this;
  }
  /**
   * Set the max number of tokens for the GPT model
   * The maximum number of to generate in the chat completion.
   * The total length of input tokens and generated tokens is limited by the model\'s context length.
   * @param maxTokens - Number of the max tokens
   */
  maxTokens(maxTokens) {
    this._maxTokens = maxTokens;
    return this;
  }
  /**
   * Set the top_p parameter for the GPT model, a value between 0 and 1.
   * An alternative to sampling with temperature, called nucleus sampling,
   * where the model considers the results of the tokens with top_p probability mass.
   * So 0.1 means only the tokens comprising the top 10% probability mass are considered.
   * We generally recommend altering this or `temperature` but not both.
   * @param topP - The top_p parameter value
   */
  topP(topP) {
    this._topP = topP;
    return this;
  }
  /**
   * Set the frequency penalty for the GPT model. Number between -2.0 and 2.0.
   * Positive values penalize new tokens based on their existing frequency in the text so far,
   * decreasing the model\'s likelihood to repeat the same line verbatim.
   * @param frequencyPenalty - The frequency penalty parameter value
   */
  frequencyPenalty(frequencyPenalty) {
    this._frequencyPenalty = frequencyPenalty;
    return this;
  }
  /**
   * Set the presence penalty for the GPT model. Number between -2.0 and 2.0.
   * Positive values penalize new tokens based on whether they appear in the text so far,
   * increasing the model\'s likelihood to talk about new topics.
   * @param presencePenalty - The presence penalty parameter value
   */
  presencePenalty(presencePenalty) {
    this._presencePenalty = presencePenalty;
    return this;
  }
  /**
   * Set the default startingObjec to add json object to the prompt
   * This is {target} JSON in promptContent
   */
  startingObject(obj) {
    this._target = obj;
    return this;
  }
  /**
   * Set the default prompt to add json object to the prompt
   *
   * Default: 'Starting function looks like this: ```{target}```.'
   * @param promptContent - The prompt to add content
   */
  promptStartingObject(promptStartingObject) {
    this._promptStartingObject = promptStartingObject;
    return this;
  }
  /**
   * Set the default prompt to add content
   *
   * Default: 'The answer should be based on this content ```{content}```.'
   * @param promptContent - The prompt to add content
   */
  promptContent(promptContent) {
    this._promptContent = promptContent;
    return this;
  }
  /**
   * Set the function name for the AI model
   *
   * Default: 'new_object_with_filled_data'
   * @param functionName - The function name
   */
  functionName(functionName) {
    this._functionName = functionName;
    this.updateFunctions();
    return this;
  }
  /**
   * Set the function description for the AI model
   *
   * Default: 'Function need all values to be filled to process.'
   * @param functionDescription - The function description
   */
  functionDescription(functionDescription) {
    this._functionDescription = functionDescription;
    this.updateFunctions();
    return this;
  }
  /**
   * Set the temperature for the GPT model. Value between 0 and 2.
   * What sampling temperature to use. Higher values like 0.8 will make the output more random,
   * while lower values like 0.2 will make it more focused and deterministic.
   * We generally recommend altering this or `top_p` but not both.
   *
   * @param temperature - The temperature parameter value
   */
  temperature(temperature) {
    this._temperature = temperature;
    return this;
  }
  /**
   * Set the content of the AI's output. It is added to the prompt as an extra message with default prompt: 'The answer should be based on this content ```{content}```.'.
   * You can override it with promptContent function.
   *
   * @param content - Content of the AI's output
   */
  content(content) {
    this._content = content;
    return this;
  }
  /**
   * Select Role or Character of the System (e.g. 'You are A software developer who can help you develop software programs and applications using programming languages and development tools. You need to call functions to complete your tasks.')
   *
   * Default: 'You are an AI assistant. You need to call functions to complete your tasks.'
   * @param system - Role or Character of the System
   */
  system(system) {
    this._system = system;
    return this;
  }
  /**
   * Sets the field schema of the target object
   * @param fieldsSchema - Field schema of the target object
   */
  schema(fieldsSchema) {
    this._fieldsSchema = fieldsSchema;
    this.updateFunctions();
    return this;
  }
  /**
   * Asynchronously populate the AI input stream using the given parameters. To get an access to the stream, get a 'stream' property.
   * @param chunkHandler - An optional callback function to handle chunks of data
   * @returns - A Promise of type `JsonGPT`
   */
  stream(chunkHandler) {
    return __async(this, null, function* () {
      var _a;
      if (!this._fieldsSchema)
        throw Error("You need to provide schema");
      if (!this._target)
        throw Error("You need to provide target");
      if (!this._prompt)
        throw Error("You need to provide prompt");
      if (((_a = this.messages) == null ? void 0 : _a.length) > 0) {
        this.messages = [...this.messages, this.prepareMessagesForNext(this._prompt)];
      } else {
        this.messages = this.prepareMessagesFirstMsg(this._prompt, this._system, this._content);
      }
      const responseReady = new Promise((resolve) => {
        let handleChunk;
        if (chunkHandler) {
          handleChunk = (chunk) => __async(this, null, function* () {
            chunkHandler(chunk);
          });
        }
        this.sendToOpenAIAPI(this.messages, this.functions, handleChunk).then((result) => {
          resolve(result);
        }).catch((err) => {
          throw Error(err);
        });
      });
      return responseReady;
    });
  }
  updateFunctions() {
    if (this._target !== void 0 && this._fieldsSchema !== void 0) {
      this.functions = [
        getFunctionJsonSchema(this._target, this._functionName, this._functionDescription, this._fieldsSchema)
      ];
    }
  }
  addPromptForStartingObject(target, originalPrompt) {
    return `${originalPrompt}. ${this._promptStartingObject.replace("{target}", JSON.stringify(target))}`;
  }
  sendToOpenAIAPI(messages, functions, chunkHandler) {
    return __async(this, null, function* () {
      if (!this._openAIApiKey)
        throw Error("You need to provide openAIApiKey");
      const payload = {
        model: this._model,
        messages,
        temperature: this._temperature,
        top_p: this._topP,
        frequency_penalty: this._frequencyPenalty,
        presence_penalty: this._presencePenalty,
        max_tokens: this._maxTokens,
        stream: true,
        function_call: { name: this._functionName },
        functions,
        user: "JsonGPT.com",
        n: 1
      };
      const jsAC = new JsonAutoComplete();
      this.streaming = yield OpenAIStream(payload, this._openAIApiKey);
      if (!chunkHandler)
        return this;
      const reader = this.streaming.getReader();
      let resultFromOpenAI = "";
      const handleChunk = (chunk) => {
        resultFromOpenAI += chunk;
        jsAC.addChunk(chunk);
        let curr = jsAC.getCorrectJson();
        chunkHandler(curr);
      };
      const decoder = new TextDecoder();
      let done = false;
      while (!done) {
        const { value, done: doneReading } = yield reader.read();
        done = doneReading;
        const chunkValue = decoder.decode(value);
        handleChunk(chunkValue);
      }
      this.messages = [...this.messages, this.prepareMessageForFunctionCallAnswer(resultFromOpenAI)];
      try {
        this.lastTarget = JSON.parse(resultFromOpenAI);
      } catch (ex) {
        new Error("Problem with parsing json object from OpenAI");
      }
      return this;
    });
  }
  prepareMessagesFirstMsg(prompt, system, content) {
    return [
      {
        role: "system",
        content: system
      },
      {
        role: "user",
        content: `${prompt} ${content ? this._promptContent.replace("{content}", content) : ""} `
      }
    ];
  }
  prepareMessagesForNext(prompt) {
    return {
      role: "user",
      content: prompt
    };
  }
  prepareMessageForFunctionCallAnswer(lastResult) {
    return {
      role: "assistant",
      content: null,
      function_call: {
        name: this._functionName,
        arguments: lastResult
      }
    };
  }
};
var jsonGPT_default = JsonGPT;

// src/utils/streamReader.ts
function streamReader(stream, callback) {
  return __async(this, null, function* () {
    let internalStream;
    if (stream instanceof Response && stream.body instanceof ReadableStream) {
      internalStream = stream.body;
    } else if (stream instanceof ReadableStream) {
      internalStream = stream;
    } else {
      throw new Error("The provided stream must be a ReadableStream or a Response object with a ReadableStream body.");
    }
    const reader = internalStream.getReader();
    const decoder = new TextDecoder();
    let result = "";
    try {
      while (true) {
        const { value, done } = yield reader.read();
        if (done) {
          return result;
        }
        if (value) {
          const chunk = decoder.decode(value, { stream: true });
          result += chunk;
          if (callback) {
            callback(result);
          }
        }
      }
    } finally {
      reader.releaseLock();
      return result;
    }
  });
}

// src/index.ts
var src_default = { JsonGPT: jsonGPT_default };
export {
  jsonGPT_default as JsonGPT,
  src_default as default,
  streamReader
};
