import { describe, it, expect } from 'vitest';
import { JsonAutoComplete } from './JsonAutoComplete'

describe('JsonAutoComplete', () => {
    it('should maintain valid JSON structure as chunks are added', () => {
      const autoComplete = new JsonAutoComplete();
      let json = '';

      autoComplete.addChunk('{');
      json = autoComplete.getCorrectJson();      
      expect(json).toBe('{}');
  
      autoComplete.addChunk('"key": "');
      json = autoComplete.getCorrectJson();
      //it should be {} because we don't know what type is key
      expect(json).toBe('{"key": ""}');
  
      autoComplete.addChunk('value"');
      json = autoComplete.getCorrectJson();
      expect(json).toBe('{"key": "value"}');
  
      autoComplete.addChunk(', "array": [');
      json = autoComplete.getCorrectJson();
      expect(json).toBe('{"key": "value", "array": []}');
  
      autoComplete.addChunk('1, 2, 3');
      json = autoComplete.getCorrectJson();
      expect(json).toBe('{"key": "value", "array": [1, 2, 3]}');
  
      autoComplete.addChunk('], "boolean": tru');
      json = autoComplete.getCorrectJson();
      //boolean is not finished so dont add it
      expect(json).toBe('{"key": "value", "array": [1, 2, 3]}');
  
      autoComplete.addChunk('e, "nullValue": nul');
      json = autoComplete.getCorrectJson();
      //null is not finished
      expect(json).toBe('{"key": "value", "array": [1, 2, 3]}');
      
      autoComplete.addChunk('l');
      json = autoComplete.getCorrectJson();
      expect(json).toBe('{"key": "value", "array": [1, 2, 3], "boolean": true, "nullValue": null}');
  
      autoComplete.addChunk('}');
      json = autoComplete.getCorrectJson();
      expect(json).toBe('{"key": "value", "array": [1, 2, 3], "boolean": true, "nullValue": null}');
    });

    // it('should handle nested objects correctly', () => {
    //     const autoComplete = new JsonAutoComplete();
    
    //     autoComplete.addChunk('{"outer": {"inner": ');
    //     let json = autoComplete.getCorrectJson();
    //     expect(json).toBe('{"outer": {}}');
    
    //     autoComplete.addChunk('42}');
    //     json = autoComplete.getCorrectJson();
    //     expect(json).toBe('{"outer": {"inner": 42}}');
    //   });

      it('should handle nested arrays correctly', () => {
        const autoComplete = new JsonAutoComplete();
    
        autoComplete.addChunk('{"nestedArray": [[1, 2], [');
        let json = autoComplete.getCorrectJson();
        expect(json).toBe('{"nestedArray": [[1, 2], []]}');
    
        autoComplete.addChunk('"a", "b"]]}');
        json = autoComplete.getCorrectJson();
        expect(json).toBe('{"nestedArray": [[1, 2], ["a", "b"]]}');
      });

      it('should not accept invalid JSON chunks', () => {
        const autoComplete = new JsonAutoComplete();
    
        // Missing opening brace
        const validChunk = autoComplete.addChunk('"invalid": true}');
        expect(validChunk).toBe(false);
    
        // JSON remains unmodified
        let json = autoComplete.getCorrectJson();
        expect(json).toBe('');
      });

      it('should handle chunks with special characters in strings', () => {
        const autoComplete = new JsonAutoComplete();
    
        autoComplete.addChunk('{"escaped": "\\"foo\\""}');
        let json = autoComplete.getCorrectJson();
        expect(json).toBe('{"escaped": "\\"foo\\""}');
      });
  });