// This type definition helps with code completion and type-checking.
type Callback = (chunk: string) => void;

export default async function streamReader(stream: ReadableStream | Response, callback?: Callback) {
  let internalStream: ReadableStream;

  // Check if stream is a Response object with a ReadableStream body.
  if (stream instanceof Response && stream.body instanceof ReadableStream) {
    internalStream = stream.body;
  } else if (stream instanceof ReadableStream) {
    internalStream = stream;
  } else {
    throw new Error('The provided stream must be a ReadableStream or a Response object with a ReadableStream body.');
  }

  const reader = internalStream.getReader();
  const decoder = new TextDecoder();
  // If no callback is provided, accumulate the chunks.
  let result = "";

  try {
    while (true) { // We'll break out of this loop when the stream is done.
      const { value, done } = await reader.read();
      if (done) {
        return result;
      }
      
      if (value) {
        const chunk = decoder.decode(value, {stream: true});
        result += chunk;
        if(callback){
          callback(result);
        }
        // if (callback) {
        //   // If a callback is provided, pass the chunk to it instead of accumulating.
        //   callback(chunk);
        // } else {
        //   // Otherwise accumulate the chunks in `result`.
        //   result += chunk;
        // }
      }
    }
    // if (!callback) {
    //   // If a callback hasn't been provided, resolve the final accumulated result.
    //   return result;
    // }
  } finally {
    reader.releaseLock();
    return result;
  }
  
}